@@ -10,14 +10,22 @@
 from datetime import date
 from dateutil.relativedelta import relativedelta
 
+from nereid.helpers import slugify
+from trytond.wizard import StateTransition
 from trytond.transaction import Transaction
 from trytond.pool import PoolMeta, Pool
 from trytond.model import fields
 from trytond.pyson import Bool, Eval
 from nereid import request, cache, jsonify, abort, current_user, route
 from nereid.helpers import key_from_list
+from nereid.contrib.locale import make_lazy_gettext
 
-__all__ = ['Product']
+_ = make_lazy_gettext('nereid_cart_b2c')
+
+__all__ = [
+    'Product', 'AddProductListingStart', 'AddProductListing',
+    'ProductChannelListing',
+]
 __metaclass__ = PoolMeta
 
 
@@ -25,6 +33,7 @@ class Product:
     "Product extension for Nereid"
     __name__ = "product.product"
 
+    # TODO: Move these fields into webshop channel listing
     display_available_quantity = fields.Boolean(
         "Display Available Quantity On Website?"
     )
@@ -71,6 +80,70 @@ class Product:
             ),
         })
 
+        # Change displayed_on_eshop to function field
+        cls.displayed_on_eshop = fields.Function(
+            fields.Boolean('Displayed on E-Shop?'),
+            'get_displayed_on_eshop', searcher="search_displayed_on_eshop"
+        )
+
+        cls.uri = fields.Function(
+            fields.Char('URI'), 'get_uri', searcher='search_uri'
+        )
+
+    @classmethod
+    def get_displayed_on_eshop(cls, products, name):
+        Listing = Pool().get('product.product.channel_listing')
+
+        res = {product.id: False for product in products}
+        listings = Listing.search([
+            ('channel.source', '=', 'webshop'),
+            ('state', '=', 'active'),
+            ('channel', '=', Transaction().context.get('current_channel')),
+            ('product', 'in', res.keys())
+        ])
+
+        for listing in listings:
+            res[listing.product.id] = True
+        return res
+
+    @classmethod
+    def search_displayed_on_eshop(cls, name, clause):
+        Listing = Pool().get('product.product.channel_listing')
+
+        listings = Listing.search([
+            ('channel.source', '=', 'webshop'),
+            ('state', '=', 'active'),
+            ('channel', '=', Transaction().context.get('current_channel')),
+        ])
+
+        return [('id', 'in', map(lambda l: l.product.id, listings))]
+
+    @classmethod
+    def get_uri(cls, products, name):
+        Listing = Pool().get('product.product.channel_listing')
+
+        res = {product.id: None for product in products}
+        listings = Listing.search([
+            ('channel.source', '=', 'webshop'),
+            ('channel', '=', Transaction().context.get('current_channel')),
+            ('product', 'in', res.keys())
+        ])
+        for listing in listings:
+            res[listing.product.id] = listing.uri
+        return res
+
+    @classmethod
+    def search_uri(cls, name, clause):
+        Listing = Pool().get('product.product.channel_listing')
+
+        listings = Listing.search([
+            ('channel.source', '=', 'webshop'),
+            ('channel', '=', Transaction().context.get('current_channel')),
+            ('uri', ) + tuple(clause[1:])
+        ])
+
+        return [('id', 'in', map(lambda l: l.product.id, listings))]
+
     @classmethod
     def validate(cls, records):
         """
@@ -103,7 +176,7 @@ class Product:
         """
         Getter for start_displaying_qty_digits
         """
-        return self.template.default_uom.digits or 2
+        return self.default_uom.digits or 2
 
     def can_buy_from_eshop(self):
         """
@@ -147,15 +220,16 @@ class Product:
         the message to show.
         """
         if self.can_buy_from_eshop():
-            status, message = 'in_stock', 'In stock'
+            status, message = 'in_stock', str(_('In stock'))
         else:
-            status, message = 'out_of_stock', 'Out of stock'
+            status, message = 'out_of_stock', str(_('Out of stock'))
 
         quantity = self.get_availability().get('quantity')
 
         if status == 'in_stock' and self.display_available_quantity and \
                 quantity <= self.start_displaying_available_quantity:
-            message = '%s %s left' % (quantity, self.default_uom.name)
+            message = '%s %s %s' % (quantity, self.default_uom.name,
+                                    str(_('left')))
 
         return status, message
 
@@ -282,3 +356,68 @@ class Product:
             return abort(404)
 
         return jsonify(product.get_availability())
+
+
+class ProductChannelListing:
+    "Product Sale Channel Listing"
+    __name__ = 'product.product.channel_listing'
+
+    uri = fields.Char(
+        'URI', select=True, states={
+            'required': Eval('channel_source') == 'webshop',
+            'invisible': Eval('channel_source') != 'webshop',
+        }, depends=['channel_source']
+    )
+
+    @fields.depends('product', 'channel', 'uri')
+    def on_change_with_uri(self):
+        """
+        If the URI is empty, slugify template name into URI
+        """
+        if self.uri:
+            return self.uri
+
+        if self.channel and self.channel.source == 'webshop' and self.product:
+            return slugify(self.product.code)
+
+
+class AddProductListingStart:
+    __name__ = 'product.listing.add.start'
+
+    uri = fields.Char(
+        'URI', select=True, states={
+            'required': Eval('channel_source') == 'webshop',
+            'invisible': Eval('channel_source') != 'webshop',
+        }, depends=['channel_source']
+    )
+
+    @fields.depends('product', 'channel', 'uri')
+    def on_change_with_uri(self):
+        """
+        If the URI is empty, slugify template name into URI
+        """
+        if self.uri:
+            return self.uri
+
+        if self.channel and self.channel.source == 'webshop' and self.product:
+            return slugify(self.product.code)
+
+    @classmethod
+    def __setup__(cls):
+        super(AddProductListingStart, cls).__setup__()
+        cls.add_source('webshop')
+
+
+class AddProductListing:
+    __name__ = 'product.listing.add'
+
+    start_webshop = StateTransition()
+
+    def transition_start_webshop(self):
+        Listing = Pool().get('product.product.channel_listing')
+        Listing.create([{
+            'product': self.start.product.id,
+            'channel': self.start.channel.id,
+            'uri': self.start.uri,
+        }])
+        return 'end'
