@@ -3,6 +3,7 @@
 # this repository contains the full copyright notices and license terms.
 import unittest
 
+from test_views_depends import TestViewsDepends
 from test_cart import TestCart
 from test_product import TestProduct
 from test_website import TestWebsite
@@ -12,6 +13,7 @@ def suite():
     "Cart test suite"
     suite = unittest.TestSuite()
     suite.addTests([
+        unittest.TestLoader().loadTestsFromTestCase(TestViewsDepends),
         unittest.TestLoader().loadTestsFromTestCase(TestCart),
         unittest.TestLoader().loadTestsFromTestCase(TestProduct),
         unittest.TestLoader().loadTestsFromTestCase(TestWebsite),
